syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface Pose 
/// \brief Message for a pose


import "vector3d.proto";
import "quaternion.proto";

message Pose
{
  optional string name            = 1;
  optional uint32 id              = 2;
  required Vector3d position      = 3;
  required Quaternion orientation = 4;
}
