syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface RestLogin
/// \brief A message to login to a REST service

message RestLogin
{
  /// \brief ID of this request message
  optional uint32 id       = 1;

  /// \brief Rest service URL
  required string url      = 2;

  /// \brief Login user name
  optional string username = 3;

  /// \brief Login password
  optional string password = 4;
}
