syntax = "proto2";
package gazebo.msgs;

/// \ingroup gazebo_msgs
/// \interface PID 
/// \brief Message for simple PID controllers


message PID
{
  optional double target = 1[default=0.0];
  optional double p_gain = 2[default=0.0];
  optional double i_gain = 3[default=0.0];
  optional double d_gain = 4[default=0.0];
  optional double i_max  = 5[default=0.0];
  optional double i_min  = 6[default=0.0];
  optional double limit  = 7[default=0.0];
}

