



if exist %PREFIX%\\Library\\bin\\gzserver.exe (exit 0) else (exit 1)
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %PREFIX%\\Library\\bin\\gzclient.exe (exit 0) else (exit 1)
IF %ERRORLEVEL% NEQ 0 exit /B 1
if exist %PREFIX%\\Library\\bin\\gazebo.dll (exit 0) else (exit 1)
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
